/** MID314 API.  API functions for the 2250 board.
 * Copyright 2006 Sensoray Company Inc.
 * @version 1.04e (May 10, 2006)
 */

#ifndef _MID314FUNC_H
#define _MID314FUNC_H

#define MAX_DEVICES 8  // maximum number of 314s in system

#include "mid314types.h"



//////////////////////////////////////////////////////////////////////
// Begin Configuration and Setup functions                          

/** Opens the SDK/API
  * @return 0 on success
  */
MID314_API int     SN_Open( void );

/** Cleans up and shuts down the API gracefully
 *  @return 0 on success
 */
MID314_API int     SN_Close( void ); 


/** Sets the video input source. 
 *  @param input is the enumerated input MID314_SOURCE enum(composite, svideo, DV) (see MID314const.h)
 *  @return 0 on success
 */
MID314_API int     SN_SetSource( MID314_SOURCE input, int board);

/** Gets the current video source
 *  @param pSource pointer to returned value of current input source
 *  @return 0 on success 
*/
MID314_API int     SN_GetSource( MID314_SOURCE *pSource, int board );

/** set the current video system.  If board is currently streaming 
 *  board will be stopped and restarted.  If board recording, recording will
 *  be stopped.
 *  @param vidsys is the video system type
 *  @return 0 on success
 */
MID314_API int     SN_SetVidSys( MIDSRAY_VIDSYS vidsys, int board );

MID314_API int     SN_GetVidSys( MIDSRAY_VIDSYS *vidsys, int board);

/** 
 *  Sets the encoding type(see MID314_ENCODING above).
 *  Calling this function will reset the IFrame only sequence mode if set.
 *
 *  @param encodeType type of encoding
 *  @return 0 on success, negative otherwise (see errors in MID314const.h)
 */
MID314_API int     SN_SetEncodeType( MIDSRAY_ENCODING encodeType, int board);

/**
 *   Retrieves the current mpeg encoding
 *
 * @return current encoding setting. -1 on error
 */
MID314_API int     SN_GetEncodeType( int board );

/** 
 *  Sets the bitrate.  Note: MAXIMUM Bitrate for half size is 2Mbit/s!
 *
 *  @param bitrate current bitrate
 *  @return 0 on success, negative otherwise (see errors in MID314const.h)
 */
MID314_API int     SN_SetBitrate( int bitrate, int board );

/**
 *   Retrieves the current bitrate
 *
 * @return current encoding setting. -1 on error
 */
MID314_API int     SN_GetBitrate(int board );


/** retrieves current SDK and file information( see MID314STATUS structure above)
 *  @return 0 on success, negative otherwise (see errors in MID314const.h)
 */
MID314_API int		SN_GetStatus( MIDSRAYSTATUS *pStatus, int board );


/** starts the preview of video and audio(if selected)
  * @param board is the board instance in the system( 0 for default or single board)
  */
MID314_API int     SN_StartPreview(int board);

/** stops video/audio stream and stops recording if recording
 */
MID314_API int     SN_StopPreview( int board);

/** set the recording mode
 *  @param recMode MID314_REC setting (see MID314types.h)
 *  @return 0 on success
 */
MID314_API int		SN_SetRecordMode( MID314_REC recMode, int board);

/** get the recording mode
 *  @param recMode MID314_REC setting (see MID314types.h)
 *  @return 0 on success
 */
MID314_API int		SN_GetRecordMode( MID314_REC *recMode, int board);

/** 
 * Starts recording to file using filter graph previously set up.
 * @param filename is the video/mux filename to record to.  Use the full path and extension
 * @return 0 on success
 */
MID314_API int     SN_StartRecord( char *filename, int board );
/** 
 * Starts recording to file using filter graph previously set up.
 * @param vidfile is the video/mux filename to record to.  Use the full path and extension
 * @param audfile is for MID314_REC_SPLIT mode only.  records separate audio channel.  
 * @return 0 on success
 */
MID314_API int     SN_StartRecordSplit( char *vidfile,char *audfile, int board );


 

/** pause recording
 */
MID314_API int		SN_PauseRecord	(int board );
/** resume recording
 */
MID314_API int		SN_ResumeRecord	(int board );

/** stops recording to file. video/sound preview window will continue
 */
MID314_API int		SN_StopRecord(int board );


///////////////////////////////////////////////////////
// Playback functions                                                    

/** plays back the specified video clip in the current window
  * default is first 
  */
MID314_API int     SN_PlaybackVideo   (char *filename_plus_path );
/** stops video playback
  * @return 0 on success 
  */
MID314_API int     SN_StopPlayback( );

/** pause/resume playback 
  * @param bPause bPause = TRUE(pause), bPause=FALSE(resume) 
  * @return 0 on success
  */
MID314_API int		SN_PausePlayback( BOOL bPause );

/** change playback rate 
 *  @param drate is a double specifying playback speed.  0.5 == half, 2.0 == double
 *  minimum speed is 0.5
 *  return 0 on success, -ve otherwise
 */
MID314_API int		SN_PlaybackSetRate( double drate );

/** seek to a position relative to the percent of total stream.
 *  @param percent is an integer from 0(start) to range(end)
 *  @param range defines the percentage granularity
 *  @return 0 on success -1 otherwise
 */
MID314_API int		SN_PlaybackSetSeekPosition( int percent, int range );

/** 
 *  retrieves current position (percent of total stream) in the playback stream
 *  @param range defines the percentage granularity
 *  @return percent, an integer from 0(start) to range(end) or -1 on error
 */
MID314_API int		SN_PlaybackGetSeekPosition( int range );


/////////////////////////////////////////////////
// General configuration options

/** set the brightness, contrast and saturation
*   @param param is the parameter to change( MID314_LEVEL_CONTRAST, MID314_LEVEL_BRIGHTNESS, MID314_LEVEL_SATURATION, MID314_LEVEL_HUE)
*   @param value is the value of the parameter
*    ( BRIGHTNESS: 0 to 255 default 128)
*    ( SATURATION: -128 to 127 default 64)
*    ( CONTRAST: -128 to 127 default 64)
*    ( HUE: -128 to 127, default 0)
*   @return 0 on success
*/
MID314_API int SN_SetLevels( int param, char value, int board );

/** get the brightness, contrast or saturation
  * @param param is the parameter to get( MID314_LEVEL_CONTRAST, MID314_LEVEL_BRIGHTNESS, MID314_LEVEL_SATURATION, MID314_LEVEL_HUE)
  * @param value is the returned value of the parameter
  *  ( CONTRAST, BRIGHTNESS, SATURATION: 0-100, default 50)
  *	 ( HUE: -50 to 50, default 0)
  * @return 0 on success
  */
MID314_API int SN_GetLevel( int param, char *value, int board );


/** 
 *  Mutes the audio on the host computer.  Audio will still be recorded.  
 *  Note: if currently streaming(StartStream used), this function will stop the stream and restart it.
 *  Default DLL mode is to have audio muted
 *  @param bMute whether to mute audio on PC
 *  @return 0 on success
 */
MID314_API int		SN_SetMute( BOOL bMute, int board);

/** 
 * Gets the mute setting
 *
 * @param bMute pointer to retrieved mute setting
 * @return 0 on success 
 */
MID314_API int		SN_GetMute( BOOL *bMute, int board);

////////////////////////////////////////////
// miscellaneous functions (GPIO, Video display,
// callback routines, message notifications

/** Set the video size.  If stream is running, this call will stop and restart it
 *  @param enum eVidSize see vidsizes( 0-- full size default, 1--half size(352x240 for NTSC))
 *  @return 0 on success
 */
MID314_API int	   SN_SetVidSize( MIDSRAY_RES eVidSize, int board);

MID314_API int     SN_GetVidSize( int *iVidSize, int board);

/** SetRenderVideo allows one to turn off the video window but still stream
 *  the video after SN_StartStream.  Video can also be recording with this setting off, but
 *  won't be displayed.
 *  @param bDisplayVideo whether to display video or not
 *  @return 0 on success
 */
MID314_API int		SN_SetRenderVideo( BOOL bDisplayVideo, int board);
/** Gets the display video setting
 *  @param bDisplayVideo pointer to display video setting
 *  @return 0 on success
 */
MID314_API int		SN_GetRenderVideo( BOOL *bDisplayVideo, int board);



////////////////////////////////////////////////////////////////////////
// Callback functions from application


/** Repaint callback should be called when application receives a WM_PAINT event.  This is necessary to
 *  notify the video renderer of a repaint event. 
 *  @param hdc device context handle or NULL if unknown
 *  @return 0 on success
 */
MID314_API int		SN_Repaint( HDC hdc);

/** Informs the video renderer that a WM_DISPLAYCHANGE message has been received by the application
 *  This callback should be called when the application has a WM_DISPLAYCHANGE event
 *  @return 0 on success
 */
MID314_API int		SN_DisplayChange();

/** Set the window for the video playback
 *  @param hwnd pointer to video display window
 *  @param reserved unused
 */
MID314_API int		SN_SetPlaybackWindow( HWND hwnd, BOOL reserved );


/** Set the window for the video streaming per board( formerly UpdateClippingWindow)
 *  @param hwnd pointer to video display window
 *  @param BOOL reserved 
 *  @param board board number
 *  Note: this setting will override SN_SetRenderVideo and turn on 
 *  Video rendering(display) as it is assumed setting a window means display is wanted
 */
MID314_API int		SN_SetBoardWindow( HWND hwnd, BOOL reserved, int board);

/** set remove msg
 *  @param messageHwnd window to notify when device removed
 *  @param removeMsg message on remove
 */

MID314_API int SN_SetRemoveMsg( HWND messageHwnd, int removeMsg, int board);


/** Test whether device was removed or not
 * When SN_SetRemoveMessage is called, a message window handle and remove message is set.
 * When remove message is received by the application, this function should be called
 * to see if the device was removed.  DLL should be shut down if device terminated
 * @return 0 if not removed, 1 if removed
 */

MID314_API int     SN_TestDeviceRemoval(int board);


/** set the video position in the clipping window 
 *  @param xpos x position of video
 *  @param ypos y position of video
 *  @param xsize size of video display
 *  @param ysize size of video display
 *  @param left_clip number of pixels at left of source video to clip
 *  @param top_clip y number of pixels at top of source video to clip
 *  @param right_clip number of pixels at right of source video to clip
 *  @param bottom_clip number of pixels at bottom of source video to clip
 *  @return 0 on success
 */ 
MID314_API int     SN_SetVideoPosition( int xpos, int ypos, int xsize, int ysize, int left_clip, int top_clip, int right_clip, int bottom_clip, int board);

MID314_API int     SN_GetNumBoards( int *pNumBoards);

#if 0
/** Advanced setting if only I frames desired in the stream.
 * 
 */
MID314_API int     SN_SetMpegSequence( MID314_SEQMODE seqMode, int board);
MID314_API int     SN_GetMpegSequence( MID314_SEQMODE *seqMode, int board);
#endif

/** returns the native video size. (only valid after streaming started) */

MID314_API int     SN_GetBoardVideoSize( RECT *rect, int board);
MID314_API int     SN_GetPlaybackVideoSize( RECT *rect);

/** takes a snapshot and saves to file in filename.  
*   @param szFilename is the fully qualified file + path( without extension)
*   @param filetype is the file type to save( see mid314types.h: MID314_FILE_JPEG and/or MID314_FILE_BMP)
*   @param qual jpeg quality, (use 25-100, default 100).
*   @param board board number
*/ 
MID314_API int SN_SnapshotToFile( const char *szFilename, int filetype, int qual, int board);


// returns raw frame from stream
// IN lSize -- size of pFrame
// OUT pBMI stores the bitmap info
//    pBMI->biWidth
//    pBMI->biHeight
//    pBMI->biBitCount
//    pBMI->biSizeImage
//    pBMI->biBitCount ( bits per pixel.  Stride == Width * biBitCount / 8)
// supports YUY2 or RGB24.
// For CaptureOnly stream, set YUY2, RGB24 type when starting graph.
// For Preview Stream, Set raw preview mode
// IN board (board number)
MID314_API int SN_GetFrame( long *inSize, unsigned char *pFrame, BITMAPINFOHEADER *pBMI, int board);



MID314_API int SN_GetMDData( int *region, unsigned char *buf, int bufsize, int board);

// SetMD starts the MD interface if event not null
// if event == NULL, then it only updates the MD regions( so they may be displayed with SN_DisplayMDRegions)

MID314_API int SN_SetMD( MIDSRAY_MDConfig_t *pMD, HANDLE event, int board);
MID314_API int SN_StopMD( int board);

MID314_API int SN_SetOverlayText(int idx, const POINT *pPos, const overlay_text_t *pOvlText, BOOL bEnable, BOOL bUpdate, int board);
MID314_API int SN_ClearOverlay( int board);


MID314_API int SN_DisplayMDRegions( BOOL bOnOff, int board);

// Enable Snapshots allows the user to turn on snapshot capabilities.
// Snapshots off by default for CPU performance.
MID314_API int SN_EnableSnapshots( BOOL bOnOff, int board);

// for MPEG1 and MPEG2 only.
// MPEG1 -- VCD compliant,  MPEG2 -- DVD compliant
// only set if burning to DVD with third party software and having difficulties.
// adds more constraints to stream.
// also IFrame only mode not available if DVD compliant set.
// Always SetDVDCompliant before setting compression type if desired DVD compliance
// Also MPEG1 VCD has different aspect ratio than regular mode.
MID314_API int SN_SetDVDCompliant( BOOL bCompliant, int board);

// Enables user to turn off AVSync.  If problems with sound card( or no sound card), 
// may need to disable AVSync(in driver) so frames are not dropped.
// NOTE: Recommended to set this before starting stream.
MID314_API int     SN_AVSync( BOOL bOnOff, int board);

// set braw to FALSE if want to view the COMPRESSED MPEG stream.
// Note. this will require MPEG decoder filters to be installed 
// AND will use more CPU.
// The advantage is you can see the MPEG overlay and gauge MPEG quality.
MID314_API int SN_SetRawPreviewMode( int braw, int board);



// Colorspace.
// RGB32 snapshots may not work.
// 1==RAW preview YUY2  // may not work for some video cards
// 2==RAW preview RGB32 // may not work for some video cards
// 3==RAW preview RGB24 // may not work for some video cards
// 4==UYVY

MID314_API int SN_SetColorSpace( int colorspace, int board);



// Note: See also SN_StartCaptureOnly_CB for the CALLBACK version(see also the console demo)
// StartCaptureOnly builds a capture graph with a connection directly to the sample grabber
// For Maximum efficiency if only SN_GetFrame frame capture is desired.
// DirectShow Graph:
//    314XBar -> 314Capture(CapturePin) -> SampleGrabber
//
// colorspace == 0, RGB24 samples (resolution set by resolution function);
// colorspace == 1, YUY2  samples (resolution set by resolution function);
MID314_API int SN_StartCaptureOnly( int colorspace, int board);

// stops capture only
MID314_API int SN_StopCaptureOnly( int board);



// 0, default, 1 Iframes only, 2 IPFrames, 3 IPB frames(MPEG1 , 2 only)
MID314_API int SN_SetMpegSequence( MID314_SEQMODE seqMode, int board);


// Uncompressed capture only 
// Builds a Directshow Filter Graph to capture UNcompressed streams using callback functions
//
// Mode = 0 (Raw)
//
//  314 -> UNcompressed stream -> samplegrabber -> null renderer
//
// Mode = 1  (Raw vid + audio(MP2) split)
//  314 -> UNcompressed stream -> samplegrabber1 -> null renderer
//  314 --> audio stream------>encoder----------> samplegrabber2 -> null renderer
//
// Mode == 2 ( Raw Vid + Raw Audio streams( 2 callbacks))
//
//  314 -> UNcompressed stream -> samplegrabber1 -> null renderer
//  314 --> audio stream------> samplegrabber2 -> null renderer
// colorspace = 0 RGB24
// colorspace = 1 YUY2
MID314_API int SN_StartCaptureOnly_CB( int recmode, int colorspace,int board);

// stops capture only
MID314_API int SN_StopCaptureOnly_CB( int board);


// For Compressed Capturing only USING callback functions
// Please see console_demo for examples of how this works
//
// Mode = 0 (VES)
//
//  314 -> compressed stream -> samplegrabber -> null renderer
//
// Mode = 1  (PES)
//  314-> compressed stream ----------->|-----|    |-------------|
//                                      | MUX | -> |SampleGrabber| -> Null Renderer
//      audio)->AudioEncoder----------->|-----|    |-------------|   
//
// Mode = 2  (PES no audio)
//  314-> compressed stream ----------->|-----|    |-------------|
//                                      | MUX | -> |SampleGrabber| -> Null Renderer
//                                      |-----|    |-------------|   
//
//
// Mode == 3 ( VES + Audio encoded split streams( 2 callbacks))
//
//  314 -> compressed stream -> samplegrabber1 -> null renderer
//  314 --> audio stream------>encoder----------> samplegrabber2 -> null renderer
//
// Mode == 4 ( VES + Raw Audio streams( 2 callbacks))  
// 
//  314 -> compressed stream -> samplegrabber1 -> null renderer
//  314 --> audio stream------> samplegrabber2 -> null renderer

MID314_API int SN_StartCompressedCaptureOnly( int mode, int board);

// stops Compressed capture only
MID314_API int SN_StopCompressedCaptureOnly( int board);

typedef int (*cbfunc_t)( BYTE *merged, long size, int board);

// This is the way to get data out of the DirectShow Filter. See the console demo
// FOR SN_StartCompressedCaptureOnly  filter graph ONLY or SN_StartCaptureOnly_CB
MID314_API int SN_RegisterVidCB( cbfunc_t callback, int board);

// for SN_StartCompressedCaptureOnly with modes 3 or 4 ONLY!
// OR SN_StartCaptureOnly_CB with modes 1 or 2
MID314_API int SN_RegisterAudCB( cbfunc_t callback, int board);


#endif
